function [metal,dislocator,CZ,pores,labeledImage]=LtGt(I,LTBool,twoPhaseBool,despeckleBool,plottingBool,picsBool)
% LtGt completes the image processing described in "a novel image
% segmentation procedure for microstructure modelling"
% Inputs:
% I an 8-bit unsigned interged image to be processed
% LTBool a bool which switches the local threshold treatment on and off
% twoPhaseBool if true the dislocator, CZ and pores are combined into the
% pores phase before output and before construction of the labeled image
% despeckBool toggles the despeckle filtering of CZLT and CZGT
% plottingBool if on plots of the output are generated
% picsBool changed the labeled image to a picture
%
% Outputs:
% metal,dislocator,CZ,pores - logical pixel maps of each phase
% labeledImage - 8bit image with all pixels belonging to the metal phase
% taking the value - 4, cz - 3, dislocator - 2 and pores - 1. This can be
% used as the input to the image based finite element solver.
%% Default values
if ~exist('LTBool','var'), LTBool=false; end
if ~exist('twoPhaseBool','var'), twoPhaseBool=false; end
if ~exist('despeckleBool','var'), despeckleBool=false; end
if ~exist('plottingBool','var'), plottingBool=false; end
if ~exist('picsBool','var'), picsBool=false; end
%remove scale bar (only for the particular application at sheffield
%universty can be set as needed)
I=I(1:960,:);
%simple threshold for image
level=graythresh(I);
%% experimenting with diferent distributions
distribution='normal';
%% find metal and dislocator peaks
[counts,x]=imhist(I);
[disMax,disIndex]=max(counts(x<(level*255)));
[metalMax,metalIndex]=max(counts(x>(level*255)));
metalIndex=round(metalIndex+level*255);
%% find centres of peaks weighted average of points
[sortedMetalCounts,sortedMetalIndex]=sort(counts(x>(level*255)),'descend');
[sortedDisCounts,sortedDisIndex]=sort(counts(x<(level*255)),'descend');

numberOfPoints=11; % Nuber of points used to find peak of curve

weightsMetal=sortedMetalCounts(1:numberOfPoints)...
    /sum(sortedMetalCounts(1:numberOfPoints));
weightsDis=sortedDisCounts(1:numberOfPoints)...
    /sum(sortedDisCounts(1:numberOfPoints));

metalCentTrue=sum(sortedMetalIndex(1:numberOfPoints).*weightsMetal);
disCentTrue=sum(sortedDisIndex(1:numberOfPoints).*weightsDis)-1;
metalCentTrue=metalCentTrue+level*255;

metalCent=round(metalCentTrue);
disCent=round(disCentTrue);
%convert from centre index to centre pixel value

%% set means and scale distribution to histogram
normalpdf = @(x,mu,sig) exp(-1*(x-mu).^2/(2*sig^2));
% difine pdfs using valuse from above
metalPdf = @(s,x) metalMax*normalpdf(x,metalCentTrue,s);
disPdf = @(s,x) disMax*normalpdf(x,disCentTrue,s);

%% Rough fit distributions to provide x0 for sigma
% create half distributions
metalpxval=I(I>=metalCent);
metalpxval2=I(I>metalCent);
metalpxval=double([metalpxval; 2*metalCent-metalpxval2]);

dispxval=I(I<=disCent&I>disCent/1.5);
dispxval2=I(I<disCent&I>disCent/1.5);
dispxval=double([dispxval; 2*disCent-dispxval2]);
% fit
pdMetal=fitdist(metalpxval,distribution);
pdDis=fitdist(dispxval,distribution);

%% least squares fitting to find sigma
metalSig=lsqcurvefit(metalPdf,pdMetal.sigma,((metalCent-5):...
    min(metalCent+10,254))',counts((metalCent-5):min(metalCent+10,254)));
disSig=lsqcurvefit(disPdf,pdDis.sigma,((disCent-10):...
    min(disCent+5,254))',counts((disCent-10):min(disCent+5,254)));
% fitted dists are below
metalPdf = @(x) metalMax*normalpdf(x,metalCentTrue,metalSig);
disPdf = @(x) disMax*normalpdf(x,disCentTrue,disSig);
%% splitting the image
x_values=0:255;
metalLine=metalPdf(x_values);
disLine=disPdf(x_values);
% material not 50% accounted for by either distribution is considered to be
% either contact zone(more below) or poroisty
accountedfor=metalLine+disLine;
unaccounted=counts'-metalLine-disLine;
%split using zero crossings nearest peaks for metal and poly
idx=find(diff(sign(unaccounted-accountedfor)));
semilevels=interp1(idx,1:length(idx),...
    [disIndex metalIndex],'linear','extrap');
porethresh=idx(floor(semilevels(1))); %between pores and dislocator
disthresh=idx(ceil(semilevels(1))); %between dislocator and cz
czthresh=idx(floor(semilevels(2))); %between cz and metal

pores=(I<=porethresh);
dislocator=(I<=disthresh)&(I>porethresh);
CZGTo=(I<=czthresh)&(I>disthresh);
metal=(I>czthresh);

%% Median filtering for LT
if LTBool
    I2=I;
    I2(dislocator|pores)=random(pdMetal,numel(I2(dislocator|pores)),1);
    h = fspecial('average', 15);
    % swap this line with next for local median
    devFromLocal=double(I2)./filter2(h, I2); 
    %devFromLocal=(double(I2)./double(medfilt2(I2,[15,15]))); %
    CZLTo=devFromLocal<0.95;
else
    CZLTo=CZGTo;
end

%% despeckle here if needed
if despeckleBool
    CZGT=bwareaopen(CZGTo, 2);
    CZLT=bwareaopen(CZLTo, 2);
    pores=bwareaopen(pores, 2);
else
    CZLT=CZLTo;
    CZGT=CZGTo;
end
%% remove layer near polyester
se=strel('rectangle',[3,3]);
nextToDis=logical(imdilate(dislocator,se)-dislocator);
CZGT(nextToDis)=0;

CZa=CZGT&CZLT;
% grow remaining cz +poly +pores to rejoin cracks to poly if they exist add
% cracks to cz if they were in original cz
ppc=(dislocator|pores)|CZa;
ppc=imdilate(ppc,se);
ppc=bwmorph(ppc,'thin',inf);

CZ=CZa|(ppc&(CZGTo|CZLTo));
CZ(dislocator)=false;
CZ=bwareaopen(CZ, 2);

% the following lines are necessary to reconnect cracks
CZ=bwmorph(CZ,'bridge',2);
CZ=bwmorph(CZ,'thin',inf);
CZ=bwmorph(CZ,'diag');

%% Reassign pixels to make sure sets are unique
CZ(dislocator)=false;
metal(CZ)=false;
CZ(pores)=false;

dislocator(~CZ&~metal&~pores&I<=level*255)=true;
metal(~CZ&~dislocator&~pores&I>level*255)=true;
%% correct if only 2 phases needed
if twoPhaseBool
    pores=pores|dislocator|CZ;
end
%% make labeled image
if picsBool
    brightness=uint8([10,63,150,215]);
else
    brightness=1:4;
end
labeledImage=I;
labeledImage(pores)=brightness(1);
labeledImage(dislocator)=brightness(2);
labeledImage(CZ)=brightness(3);
labeledImage(metal)=brightness(4);
%% showing results
if plottingBool
    % showing fitted histograms
    figure, imhist(I);
    hold on
    plot(x_values,metalLine,'r','LineWidth',2)
    plot(x_values,disLine,'r','LineWidth',2)
    plot(x_values,counts'-metalLine-disLine,'r','LineWidth',2)
    %showing segmented and origninal image
    figure,imshow(labeledImage)
    colormap('jet')
    figure,imshow(I)
    colormap('jet')
    % showing resulting histograms
    figure, hist(I(metal),0:255)
    hold on
    hist(I(dislocator),0:255,'b')
    hist(I(CZ),0:255,'y')
    hist(I(pores),0:255,'k')
end
%% check that all pixels are in only one set
if max(max(metal+dislocator+CZ+pores))~=min(min(metal+dislocator+CZ+pores))
    error('somthing is broken with the image analysis');
end