function [conductivity, specificHeat, density]=small_FE(I, domainSize, XorY, periodicBool, czConductivity)
% small_FE(I, domainSize, XorY) performs finite element analysis on a sub
% domain of the image I, the size of this domain is a square of domainSize
% by domainSize. The option XorY decides the direction the emperature
% gradient will be applied, this is a sting input.
%
% The image I should be a segmented labeled image with 1 refering to pixels
% form the porosity phase, 2 for the polyester or hBN phase, 3 for the
% contact zone or crack phase and 4 for the metalic phase.
%
% The material properties used for each material are listed in the
% materials section of this code and can be edited as required by the user
%
% The image is converted to a mesh of 4 node quadriateral elements each
% refering to a single pixel for the original image. The elements are then
% given the properties for the pixel they represent.
%
% The model is solved by the backslash operator which has been tested for
% images upto 900 by 900 pixels with no problems.
% For larger problems it can be replaced by the lines below:
% prCond = ichol(GSM,struct('michol','on'));
% temperature=pcg(GSM,f,1e-8,200,prCond,prCond');
%
% The temperature results are post processed to give the average
% temperature gradient across each element, this is then multiplied by the
% element conductivity to give elemental heat flux. This is summed across
% the dimention perpendicular to the applied temperature gradient to give
% total heat flux through the model.
%
% As a square domain is used this is related back to the conductivity of
% the model by dividing the total heat flux by the applied temperature
% gradient.
%
% The Density and specifc heat capacity are found as a mean of the values
% for the pixels in the image

%% read in image
%I=imread('quarterseg1.png');
%% preliminaries
if nargin < 2
    XorY = 'y';
    domainSize = min(size(I));
elseif nargin < 3
    XorY = 'y';
end
elementWidth = 1;
%% select small area of image
imageSection=I(1:domainSize, 1:domainSize);
if (XorY=='Y' || XorY=='y')
    imageSection=imageSection';
end
%% materials
% material order is [pores poly CZ metal]
%320
conductivities = [0.0257 0.02 1.3 130];
specificHeats = [1005 808 910 910]; 
densities = [1.205 2100 2700 2700];
%601
% conductivities = [0.0257 0.15 1.3 130];
% specificHeats = [1005 1350 910 910]; 
% densities = [1.205 1400 2700 2700];

if nargin==5
    conductivities(3)=czConductivity;
end

elementConductivities=conductivities(imageSection);
elementSpecificHeats=specificHeats(imageSection);
elementDensities=densities(imageSection);
%% mesh 
% nodes number in order: [tl tr br bl]
nElements = numel(imageSection);
nNodes = nElements+2*domainSize+1;
connectivity = zeros(nElements, 4);
firstNode = floor(1:(domainSize+1)/(domainSize):nElements+domainSize);
connectivity(:, 1) = firstNode;
connectivity(:, 2) = firstNode+1;
connectivity(:, 3) = firstNode+domainSize+2;
connectivity(:, 4) = firstNode+domainSize+1;
%% element siffness matrix (square element)
% multiply by conductivity to give value for each element
kel = 1/6/elementWidth*  [4 -1 -2 -1
                        -1 4 -1 -2
                        -2 -1 4 -1
                        -1 -2 -1 4];
%% global stiffness matrix
% sparse automatically adds values together if two are assigned at the same
% time (perfect behaviour for this)
% make row, col, and data vectors, use sparse once at the end
allRows = zeros(nElements*16, 1);
allCols = zeros(nElements*16, 1);
allStiff = zeros(nElements*16, 1);
for iElement = 1 : nElements
    eleNodes = connectivity(iElement, :);
    allRows(((iElement-1)*16+1):(iElement*16)) = repmat(eleNodes, 1, 4);
    allCols(((iElement-1)*16+1):(iElement*16)) = repmat(eleNodes, 4, 1);
    allStiff(((iElement-1)*16+1):(iElement*16)) = ...
        kel*elementConductivities(iElement);
end
K = sparse(allRows, allCols, allStiff, nNodes, nNodes);
%% boundry conditions
% constant temp boundry
temperature = zeros(nNodes, 1);
f = zeros(nNodes, 1);
deltaT = 100;
leftBoundryNodes = (1:domainSize+1)';
rightBoundryNodes = (nNodes-domainSize:nNodes)';
leftVlaues = deltaT*ones(size(leftBoundryNodes));
rightValues = zeros(size(rightBoundryNodes));
fixedDofs = [leftBoundryNodes; rightBoundryNodes];
fixedDofValues = [leftVlaues; rightValues];
freeDofs = 1:nNodes;
freeDofs(fixedDofs) = [];
%% recondition K to accept essential boundry conditions
% from section 5.3 http://ms.mcmaster.ca/~bprotas/MATH745b/matlab_fem.pdf
bcwt = trace(K)/nNodes;
f = f - K(:, fixedDofs)*fixedDofValues;
K(:, fixedDofs) = 0;
K(fixedDofs, :) = 0;
K(fixedDofs, fixedDofs) = bcwt*speye(length(fixedDofs));
f(fixedDofs) = bcwt*fixedDofValues;
%% Rearange for periodic boundry
if periodicBool
    topNodes=1:(domainSize+1):nNodes;
    bottomNodes=(domainSize+1):(domainSize+1):nNodes;
    interiorNodes=1:nNodes;
    interiorNodes([topNodes,bottomNodes])=[];
    nINodes=length(interiorNodes);
    nTNodes=length(topNodes);
    nBNodes=length(bottomNodes);
    K2=sparse(nINodes+nBNodes,nINodes+nBNodes);
    K2(1:nINodes,1:nINodes)=K(interiorNodes,interiorNodes);
    K2((nINodes+1):(nINodes+nTNodes),1:nINodes)=K(topNodes,interiorNodes)+K(bottomNodes,interiorNodes);
    K2(1:nINodes,(nINodes+1):(nINodes+nTNodes))=K(interiorNodes,topNodes)+K(interiorNodes,bottomNodes);
    K2((nINodes+1):(nINodes+nTNodes),(nINodes+1):(nINodes+nTNodes))=K(topNodes,topNodes)+K(bottomNodes,bottomNodes)+K(topNodes,bottomNodes)+K(bottomNodes,topNodes);
    f2=zeros([nBNodes+nINodes,1]);
    f2(1:nINodes)=f(interiorNodes);
    f2((nINodes+1):(nINodes+nBNodes))=f(topNodes)+f(bottomNodes);
%% solving
    U = f2'/K2;
    temperature2(interiorNodes)=U(1:nINodes);
    temperature2([topNodes,bottomNodes])=[U((nINodes+1):(nINodes+nBNodes)), U((nINodes+1):(nINodes+nBNodes))];
else
    temperature2 = f'/K;
end
temperature(fixedDofs) = fixedDofValues;
temperature(freeDofs) = temperature2(freeDofs);
temperature = reshape(temperature, [domainSize+1, domainSize+1]);
%temperature = temperature';
%% Post processing for elemental heat flux
difX = diff(temperature, 1, 2);
elementTempGradX = (difX(1:end-1, :)+diff(difX, 1, 1)/2)/elementWidth;
elementHeatFluxX = -1*elementTempGradX.*elementConductivities;
conductivity = mean(sum(elementHeatFluxX, 2))/deltaT;
%% plotting result
% image(temperature);
% figure, imshow((imageSection*50));
% figure, image(elementHeatFluxX)
%% Specific heat and density
specificHeat=mean(elementSpecificHeats(:));
density=mean(elementDensities(:));
end